@@ -1,6 +1,7 @@
-"""
-The :mod:`sklearn.manifold` module implements data embedding techniques.
-"""
+"""Data embedding techniques."""
+
+# Authors: The scikit-learn developers
+# SPDX-License-Identifier: BSD-3-Clause
 
 from ._isomap import Isomap
 from ._locally_linear import LocallyLinearEmbedding, locally_linear_embedding
@@ -9,13 +10,13 @@ from ._spectral_embedding import SpectralEmbedding, spectral_embedding
 from ._t_sne import TSNE, trustworthiness
 
 __all__ = [
-    "locally_linear_embedding",
-    "LocallyLinearEmbedding",
-    "Isomap",
     "MDS",
-    "smacof",
-    "SpectralEmbedding",
-    "spectral_embedding",
     "TSNE",
+    "Isomap",
+    "LocallyLinearEmbedding",
+    "SpectralEmbedding",
+    "locally_linear_embedding",
+    "smacof",
+    "spectral_embedding",
     "trustworthiness",
 ]
