@@ -1,27 +1,26 @@
-import warnings
-import numpy as np
-import pickle
 import copy
+import pickle
+import warnings
 
+import numpy as np
 import pytest
+from scipy.special import expit
 
+import sklearn
 from sklearn.datasets import make_regression
 from sklearn.isotonic import (
-    check_increasing,
-    isotonic_regression,
     IsotonicRegression,
     _make_unique,
-)
-
-from sklearn.utils.validation import check_array
-from sklearn.utils._testing import (
-    assert_allclose,
-    assert_array_equal,
-    assert_array_almost_equal,
+    check_increasing,
+    isotonic_regression,
 )
 from sklearn.utils import shuffle
-
-from scipy.special import expit
+from sklearn.utils._testing import (
+    assert_allclose,
+    assert_array_almost_equal,
+    assert_array_equal,
+)
+from sklearn.utils.validation import check_array
 
 
 def test_permutation_invariance():
@@ -228,7 +227,13 @@ def test_isotonic_regression_with_ties_in_differently_sized_groups():
 
 def test_isotonic_regression_reversed():
     y = np.array([10, 9, 10, 7, 6, 6.1, 5])
+    y_result = np.array([10, 9.5, 9.5, 7, 6.05, 6.05, 5])
+
+    y_iso = isotonic_regression(y, increasing=False)
+    assert_allclose(y_iso, y_result)
+
     y_ = IsotonicRegression(increasing=False).fit_transform(np.arange(len(y)), y)
+    assert_allclose(y_, y_result)
     assert_array_equal(np.ones(y_[:-1].shape), ((y_[:-1] - y_[1:]) >= 0))
 
 
@@ -503,25 +508,25 @@ def test_isotonic_copy_before_fit():
     copy.copy(ir)
 
 
-def test_isotonic_dtype():
+@pytest.mark.parametrize("dtype", [np.int32, np.int64, np.float32, np.float64])
+def test_isotonic_dtype(dtype):
     y = [2, 1, 4, 3, 5]
     weights = np.array([0.9, 0.9, 0.9, 0.9, 0.9], dtype=np.float64)
     reg = IsotonicRegression()
 
-    for dtype in (np.int32, np.int64, np.float32, np.float64):
-        for sample_weight in (None, weights.astype(np.float32), weights):
-            y_np = np.array(y, dtype=dtype)
-            expected_dtype = check_array(
-                y_np, dtype=[np.float64, np.float32], ensure_2d=False
-            ).dtype
+    for sample_weight in (None, weights.astype(np.float32), weights):
+        y_np = np.array(y, dtype=dtype)
+        expected_dtype = check_array(
+            y_np, dtype=[np.float64, np.float32], ensure_2d=False
+        ).dtype
 
-            res = isotonic_regression(y_np, sample_weight=sample_weight)
-            assert res.dtype == expected_dtype
+        res = isotonic_regression(y_np, sample_weight=sample_weight)
+        assert res.dtype == expected_dtype
 
-            X = np.arange(len(y)).astype(dtype)
-            reg.fit(X, y_np, sample_weight=sample_weight)
-            res = reg.predict(X)
-            assert res.dtype == expected_dtype
+        X = np.arange(len(y)).astype(dtype)
+        reg.fit(X, y_np, sample_weight=sample_weight)
+        res = reg.predict(X)
+        assert res.dtype == expected_dtype
 
 
 @pytest.mark.parametrize("y_dtype", [np.int32, np.int64, np.float32, np.float64])
@@ -596,7 +601,7 @@ def test_isotonic_thresholds(increasing):
     # the data is already strictly monotonic which is not the case with
     # this random data)
     assert X_thresholds.shape[0] < X.shape[0]
-    assert np.in1d(X_thresholds, X).all()
+    assert np.isin(X_thresholds, X).all()
 
     # Output thresholds lie in the range of the training set:
     assert y_thresholds.max() <= y.max()
@@ -680,3 +685,24 @@ def test_get_feature_names_out(shape):
     assert isinstance(names, np.ndarray)
     assert names.dtype == object
     assert_array_equal(["isotonicregression0"], names)
+
+
+def test_isotonic_regression_output_predict():
+    """Check that `predict` does return the expected output type.
+
+    We need to check that `transform` will output a DataFrame and a NumPy array
+    when we set `transform_output` to `pandas`.
+
+    Non-regression test for:
+    https://github.com/scikit-learn/scikit-learn/issues/25499
+    """
+    pd = pytest.importorskip("pandas")
+    X, y = make_regression(n_samples=10, n_features=1, random_state=42)
+    regressor = IsotonicRegression()
+    with sklearn.config_context(transform_output="pandas"):
+        regressor.fit(X, y)
+        X_trans = regressor.transform(X)
+        y_pred = regressor.predict(X)
+
+    assert isinstance(X_trans, pd.DataFrame)
+    assert isinstance(y_pred, np.ndarray)
