@@ -9,10 +9,8 @@ near-duplicates to form the final set of centroids.
 Seeding is performed using a binning technique for scalability.
 """
 
-# Authors: Conrad Lee <conradlee@gmail.com>
-#          Alexandre Gramfort <alexandre.gramfort@inria.fr>
-#          Gael Varoquaux <gael.varoquaux@normalesup.org>
-#          Martino Sorbaro <martino.sorbaro@ed.ac.uk>
+# Authors: The scikit-learn developers
+# SPDX-License-Identifier: BSD-3-Clause
 
 import warnings
 from collections import defaultdict
@@ -27,7 +25,7 @@ from ..neighbors import NearestNeighbors
 from ..utils import check_array, check_random_state, gen_batches
 from ..utils._param_validation import Interval, validate_params
 from ..utils.parallel import Parallel, delayed
-from ..utils.validation import check_is_fitted
+from ..utils.validation import check_is_fitted, validate_data
 
 
 @validate_params(
@@ -84,7 +82,7 @@ def estimate_bandwidth(X, *, quantile=0.3, n_samples=None, random_state=0, n_job
     >>> X = np.array([[1, 1], [2, 1], [1, 0],
     ...               [4, 7], [3, 5], [3, 6]])
     >>> estimate_bandwidth(X, quantile=0.5)
-    1.61...
+    np.float64(1.61)
     """
     X = check_array(X)
 
@@ -122,7 +120,7 @@ def _mean_shift_single_seed(my_mean, X, nbrs, max_iter):
         my_mean = np.mean(points_within, axis=0)
         # If converged or at max_iter, adds the cluster
         if (
-            np.linalg.norm(my_mean - my_old_mean) < stop_thresh
+            np.linalg.norm(my_mean - my_old_mean) <= stop_thresh
             or completed_iterations == max_iter
         ):
             break
@@ -218,8 +216,8 @@ def mean_shift(
 
     Notes
     -----
-    For an example, see :ref:`examples/cluster/plot_mean_shift.py
-    <sphx_glr_auto_examples_cluster_plot_mean_shift.py>`.
+    For a usage example, see
+    :ref:`sphx_glr_auto_examples_cluster_plot_mean_shift.py`.
 
     Examples
     --------
@@ -229,8 +227,8 @@ def mean_shift(
     ...               [4, 7], [3, 5], [3, 6]])
     >>> cluster_centers, labels = mean_shift(X, bandwidth=2)
     >>> cluster_centers
-    array([[3.33..., 6.     ],
-           [1.33..., 0.66...]])
+    array([[3.33, 6.     ],
+           [1.33, 0.66]])
     >>> labels
     array([1, 1, 1, 0, 0, 0])
     """
@@ -310,6 +308,9 @@ class MeanShift(ClusterMixin, BaseEstimator):
 
     Seeding is performed using a binning technique for scalability.
 
+    For an example of how to use MeanShift clustering, refer to:
+    :ref:`sphx_glr_auto_examples_cluster_plot_mean_shift.py`.
+
     Read more in the :ref:`User Guide <mean_shift>`.
 
     Parameters
@@ -431,6 +432,9 @@ class MeanShift(ClusterMixin, BaseEstimator):
     array([1, 0])
     >>> clustering
     MeanShift(bandwidth=2)
+
+    For a comparison of Mean Shift clustering with other clustering algorithms, see
+    :ref:`sphx_glr_auto_examples_cluster_plot_cluster_comparison.py`
     """
 
     _parameter_constraints: dict = {
@@ -479,7 +483,7 @@ class MeanShift(ClusterMixin, BaseEstimator):
         self : object
                Fitted instance.
         """
-        X = self._validate_data(X)
+        X = validate_data(self, X)
         bandwidth = self.bandwidth
         if bandwidth is None:
             bandwidth = estimate_bandwidth(X, n_jobs=self.n_jobs)
@@ -570,6 +574,6 @@ class MeanShift(ClusterMixin, BaseEstimator):
             Index of the cluster each sample belongs to.
         """
         check_is_fitted(self)
-        X = self._validate_data(X, reset=False)
+        X = validate_data(self, X, reset=False)
         with config_context(assume_finite=True):
             return pairwise_distances_argmin(X, self.cluster_centers_)
